﻿using System.Windows;
using System.Windows.Input;
using Demo.WpfApp.Models;
using Infragistics.Windows.Chart;

namespace Demo.WpfApp.Helpers
{

	public class XamChartHelper
	{

		#region Handle XamChart Click 'n Drag events

		private static bool started;
		private static DataPoint selectedDataPoint;

		public static readonly DependencyProperty ChartDraggingProperty =
			DependencyProperty.RegisterAttached("ChartDragging", typeof(bool), typeof(XamChartHelper),
																					new PropertyMetadata(new PropertyChangedCallback(ChartDragHandler)));

		public static bool GetChartDragging(DependencyObject obj)
		{
			return (bool) obj.GetValue(ChartDraggingProperty);
		}

		public static void SetChartDragging(DependencyObject obj, bool value)
		{
			obj.SetValue(ChartDraggingProperty, value);
		}

		private static void ChartDragHandler(DependencyObject obj, DependencyPropertyChangedEventArgs e)
		{

			var xamChart = obj as XamChart;
			if ( xamChart == null )
				return;

			if ( (bool) e.NewValue )
			{
				xamChart.MouseLeftButtonDown += MouseLeftButtonDownHandler;
				xamChart.MouseLeftButtonUp += MouseLeftButtonUpHandler;
				xamChart.MouseMove += MouseMoveHandler;
				xamChart.MouseLeave += MouseLeave;
			}
			else
			{
				xamChart.MouseLeftButtonDown -= MouseLeftButtonDownHandler;
				xamChart.MouseLeftButtonUp -= MouseLeftButtonUpHandler;
				xamChart.MouseMove -= MouseMoveHandler;
				xamChart.MouseLeave -= MouseLeave;
			}

		}

		private static void MouseLeave(object sender, MouseEventArgs e)
		{
			UpdateDataObject(sender);
			e.Handled = true;
		}

		private static void MouseMoveHandler(object sender, MouseEventArgs e)
		{

			var xamChart = sender as XamChart;
			if ( xamChart == null || started == false || selectedDataPoint == null )
				return;

			var y = FormatValue(xamChart.HitTest(e).YValue);
			selectedDataPoint.Value = y;

		}

		private static void MouseLeftButtonUpHandler(object sender, MouseButtonEventArgs e)
		{
			UpdateDataObject(sender);
			e.Handled = true;
		}

		private static void UpdateDataObject(object sender)
		{

			var xamChart = sender as XamChart;
			if ( xamChart == null )
				return;

			if ( started )
			{
				xamChart.Cursor = Cursors.Arrow;
				started = false;

				if ( selectedDataPoint == null )
					return;

				var dataItem = selectedDataPoint.DataContext as XamChartDataItem;
				if ( dataItem == null )
					return;
				
				dataItem.Value = selectedDataPoint.Value;
				selectedDataPoint.ToolTip = dataItem.Tooltip;

			}
			selectedDataPoint = null;

		}

		private static void MouseLeftButtonDownHandler(object sender, MouseButtonEventArgs e)
		{

			var xamChart = sender as XamChart;
			if ( xamChart == null )
				return;

			var hitArgs = xamChart.HitTest(e);
			selectedDataPoint = hitArgs.SelectedObject as DataPoint;
			if ( selectedDataPoint != null )
			{
				xamChart.Cursor = Cursors.SizeNS;
				started = true;
			}

			e.Handled = true;

		}

		private static double FormatValue(double value)
		{
			return double.Parse(string.Format(value.ToString("N0")));
		}

		#endregion Handle XamChart Click 'n Drag events

	}

}
