﻿using System.Collections;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Data;
using Infragistics.Windows.Chart;

namespace Demo.WpfApp.Helpers
{
	public class XamChartSeriesBindingInfo : FrameworkElement
	{
		private XamChart targetChart;
		public XamChart TargetChart
		{
			get
			{
				return targetChart;
			}
			set
			{
				targetChart = value;
				//bridge the data context from the parent hierarchy.
				var binding = new Binding("DataContext")
				{
					Source = targetChart
				};
				SetBinding(DataContextProperty, binding);
				Clear();
				UpdateFromCollection(ItemsSource);
			}
		}

		public static DependencyProperty ItemsSourceProperty = DependencyProperty.Register("ItemsSource", typeof(IEnumerable), typeof(XamChartSeriesBindingInfo), new FrameworkPropertyMetadata(null, (o, e) => ( (XamChartSeriesBindingInfo) o ).OnItemsSourceChanged(e)));

		public IEnumerable ItemsSource
		{
			get
			{
				return (IEnumerable) GetValue(ItemsSourceProperty);
			}
			set
			{
				SetValue(ItemsSourceProperty, value);
			}
		}

		public static DependencyProperty SeriesTemplateProperty = DependencyProperty.Register("SeriesTemplate", typeof(Series), typeof(XamChartSeriesBindingInfo), new FrameworkPropertyMetadata(null, (o, e) => ( (XamChartSeriesBindingInfo) o ).OnSeriesTemplateChanged(e)));

		public Series SeriesTemplate
		{
			get
			{
				return (Series) GetValue(SeriesTemplateProperty);
			}
			set
			{
				SetValue(SeriesTemplateProperty, value);
			}
		}

		protected void OnItemsSourceChanged(DependencyPropertyChangedEventArgs e)
		{

			Clear();

			if ( e.OldValue != null )
			{
				UnRegCollection(e.OldValue as IEnumerable);
			}

			if ( e.NewValue != null )
			{
				RegCollection(e.NewValue as IEnumerable);
				UpdateFromCollection(e.NewValue as IEnumerable);
			}

		}

		protected void OnSeriesTemplateChanged(DependencyPropertyChangedEventArgs e)
		{
			Clear();
			UpdateFromCollection(ItemsSource);
		}

		private void Clear()
		{
			if ( TargetChart == null )
				return;
			TargetChart.Series.Clear();
		}

		private void UpdateFromCollection(IEnumerable collection)
		{

			if ( collection == null )
				return;

			foreach ( object item in collection )
				AddItem(item);

		}

		private void AddItem(object item)
		{
			if ( SeriesTemplate == null )
				return;

			var series = new Series
			{
				DataContext = item
			};

			SetSeriesBindings(series);
			TargetChart.Series.Add(series);
		}

		private void SetSeriesBindings(Series series)
		{
			if ( SeriesTemplate == null )
				return;

			foreach ( var dp in XamChartSeriesDependencyPropertyEnumerator.DependencyProperties(typeof(Series)) )
			{

				object value;
				BindingExpression be;

				if ( XamChartSeriesDependencyPropertyEnumerator.DependencyPropertyHasBinding(SeriesTemplate, dp, out be) )
					series.SetBinding(dp, be.ParentBinding);
				else if ( XamChartSeriesDependencyPropertyEnumerator.DependencyPropertyHasValue(SeriesTemplate, dp, out value) )
					series.SetValue(dp, value);

			}

		}

		private void RegCollection(IEnumerable collection)
		{
			if ( collection is INotifyCollectionChanged )
				( collection as INotifyCollectionChanged ).CollectionChanged += SeriesBindingInfoCollectionChanged;
		}

		private void UnRegCollection(IEnumerable collection)
		{
			if ( collection is INotifyCollectionChanged )
				( collection as INotifyCollectionChanged ).CollectionChanged -= SeriesBindingInfoCollectionChanged;
		}

		void SeriesBindingInfoCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			if ( TargetChart == null )
				return;
			Clear();
			UpdateFromCollection(ItemsSource);
		}

	}

}
