﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Windows;
using System.Windows.Data;
using Infragistics.Windows.Chart;

namespace Demo.WpfApp.Helpers
{

	public class XamChartSeriesDependencyPropertyEnumerator
	{

		public static IEnumerable<DependencyProperty> DependencyProperties(Type type)
		{
			var props = from prop in type.GetProperties(BindingFlags.Instance | BindingFlags.Public | BindingFlags.FlattenHierarchy)
									join field in type.GetFields(BindingFlags.Static | BindingFlags.Public | BindingFlags.FlattenHierarchy)
											on prop.Name + "Property" equals field.Name into propGroup
									from item in propGroup.DefaultIfEmpty(null)
									where item != null
									select (DependencyProperty) item.GetValue(null);
			return props;
		}

		public static bool DependencyPropertyHasBinding(FrameworkContentElement obj, DependencyProperty dp, out BindingExpression be)
		{

			be = null;
			if ( obj == null )
				return false;

			be = obj.GetBindingExpression(dp);
			return be != null;

		}

		public static bool DependencyPropertyHasValue(FrameworkContentElement obj, DependencyProperty dp, out object value)
		{

			value = null;

			if ( obj == null )
				return false;

			var localValue = obj.ReadLocalValue(dp);
			var defaultValue = dp.GetMetadata(typeof(Series)).DefaultValue;

			if ( localValue != null && localValue != DependencyProperty.UnsetValue && !localValue.Equals(defaultValue) )
			{
				value = localValue;
				return true;
			}
			return false;

		}

	}
}
