﻿using System.Collections.Generic;
using System.Windows;
using Infragistics.Windows.DataPresenter;

namespace Demo.WpfApp.Helpers
{

	public class XamDataGridHelper : DependencyObject
	{

		public static readonly DependencyProperty FieldsProperty = DependencyProperty.RegisterAttached("Fields", typeof(List<Field>), typeof(XamDataGridHelper), new PropertyMetadata(new PropertyChangedCallback(FieldsChanged)));

		public static List<Field> GetFields(DependencyObject obj)
		{
			return (List<Field>)obj.GetValue(FieldsProperty);
		}

		public static void SetFields(DependencyObject obj, List<Field> value)
		{
			obj.SetValue(FieldsProperty, value);
		}

		private static void FieldsChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
		{
			var grid = obj as XamDataGrid;
			var fieldCollection = e.NewValue as List<Field>;

			if (grid == null || fieldCollection == null)
				return;

			if (grid.FieldLayouts == null || grid.FieldLayouts.Count == 0)
			{
				if (grid.FieldLayouts != null) 
					grid.FieldLayouts.Add(new FieldLayout());
			}
			else
			{
				grid.FieldLayouts[0].Fields.Clear();
			}

			if (grid.FieldLayouts != null)
				fieldCollection.ForEach(grid.FieldLayouts[0].Fields.Add);

		}

	}

}
