﻿using System.Collections.Generic;
using System.Collections.ObjectModel;

using Iwtnw.Lib.SampleData.Models;

namespace Demo.WpfApp.Models
{
	public class BorderCrossingChartingDto : ObservableCollection<KeyValuePair<string, int>>
	{

		public string Title { get; set; }
		public string State { get; set; }
		public string Port { get; set; }
		public int Year { get; set; }
		public string Month { get; set; }
		public int Buses { get; set; }
		public int BusPassengers { get; set; }
		public int Trucks { get; set; }
		public int EmptyTruckContainers { get; set; }
		public int LoadedTruckContainers { get; set; }
		public int Trains { get; set; }
		public int EmptyRailContainers { get; set; }
		public int LoadedRailContainers { get; set; }
		public int PersonalVehicles { get; set; }
		public int PersonalVehiclePassengers { get; set; }
		public int Pedestrians { get; set; }
		public int TotalVehicles { get { return Buses + Trucks + Trains + PersonalVehicles; } }
		public int TotalIndividuals { get { return BusPassengers + PersonalVehiclePassengers + Pedestrians; } }

		public ObservableCollection<KeyValuePair<string, int>> Data { get { return this; } }

		public BorderCrossingChartingDto(BorderCrossing borderCrossing)
		{
			
			State = borderCrossing.State;
			Port = borderCrossing.Port;
			Year = borderCrossing.Year;
			Month = borderCrossing.Month;

			Title = Port + ", " + State + " " + Month + ", " + Year;

			Buses = borderCrossing.Buses;
			BusPassengers = borderCrossing.BusPassengers;
			Trucks = borderCrossing.Trucks;
			EmptyTruckContainers = borderCrossing.EmptyTruckContainers;
			LoadedTruckContainers = borderCrossing.LoadedTruckContainers;
			Trains = borderCrossing.Trains;
			EmptyRailContainers = borderCrossing.EmptyRailContainers;
			LoadedRailContainers = borderCrossing.LoadedRailContainers;
			PersonalVehicles = borderCrossing.PersonalVehicles;
			PersonalVehiclePassengers = borderCrossing.PersonalVehiclePassengers;
			Pedestrians = borderCrossing.Pedestrians;

			AddItems();
		}

		public BorderCrossingChartingDto Refresh()
		{
			Clear();
			AddItems();
			return this;
		}

		private void AddItems()
		{
			Add(new KeyValuePair<string, int>("Buses", Buses));
			Add(new KeyValuePair<string, int>("Bus Passengers", BusPassengers));
			Add(new KeyValuePair<string, int>("Trucks", Trucks));
			Add(new KeyValuePair<string, int>("Empty Trucks", EmptyTruckContainers));
			Add(new KeyValuePair<string, int>("Loaded Trucks", LoadedTruckContainers));
			Add(new KeyValuePair<string, int>("Trains", Trains));
			Add(new KeyValuePair<string, int>("Empty Rail Containers", EmptyRailContainers));
			Add(new KeyValuePair<string, int>("Loaded Rail Containers", LoadedRailContainers));
			Add(new KeyValuePair<string, int>("Personal Vehicles", PersonalVehicles));
			Add(new KeyValuePair<string, int>("Personal Vehicle Passengers", PersonalVehiclePassengers));
			Add(new KeyValuePair<string, int>("Pedestrians", Pedestrians));
		}
	}

}
