﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using Demo.WpfApp.Commands;
using Demo.WpfApp.Models;
using Infragistics.Windows.Chart;
using Infragistics.Windows.Controls;
using Infragistics.Windows.DataPresenter;
using System.Reflection;

using Iwtnw.Lib.SampleData.Models;
using Iwtnw.Lib.SampleData.Repositories;

namespace Demo.WpfApp.ViewModels
{

	public class MainViewModel : ViewModelBase
	{

		private ICommand exitCommand;

		private BorderCrossingRepository borderCrossingRepository;

		private string chartTitle;
		public string ChartTitle
		{
			get
			{
				return chartTitle;
			}
			set
			{
				chartTitle = value;
				OnPropertyChanged("ChartTitle");
			}
		}

		public string Version
		{
			get
			{
				return VersionMethod();
			}
		}

		public ObservableCollection<XamChartSeriesItem> TestSeriesItemCollection
		{
			get;
			set;
		}

		public FieldLayoutSettings FieldLayoutSettings
		{
			get;
			set;
		}

		public FieldSettings FieldSettings
		{
			get;
			set;
		}

		private BorderCrossing currentBorderCrossing;
		public BorderCrossing CurrentBorderCrossing
		{

			get
			{
				return currentBorderCrossing;
			}

			set
			{
				if ( currentBorderCrossing == value )
					return;

				currentBorderCrossing = value;
				OnPropertyChanged("CurrentBorderCrossing");

				ChartTitle = currentBorderCrossing.Port + ", " + currentBorderCrossing.State;
				OnPropertyChanged("ChartTitle");

				UpdateBorderCrossingDetails();
			}

		}

		public BorderCrossingChartingDto BorderCrossingChartingDto
		{
			get;
			private set;
		}

		public ObservableCollection<BorderCrossingChartingDto> BorderCrossingChartingDtoCollection
		{
			get;
			private set;
		}

		private ObservableCollection<BorderCrossing> borderCrossingCollection;
		public ObservableCollection<BorderCrossing> BorderCrossingCollection
		{

			get
			{
				return borderCrossingCollection;
			}

			set
			{

				if ( borderCrossingCollection != value )
				{
					borderCrossingCollection = value;
					OnPropertyChanged("BorderCrossingCollection");
				}
			}

		}

		private List<Field> fieldCollection;
		public List<Field> FieldCollection
		{

			get
			{
				return fieldCollection;
			}

			set
			{
				if (fieldCollection == value)
					return;

				fieldCollection = value;
				OnPropertyChanged("FieldCollection");
			}

		}

		public ICommand ExitCommand
		{
			get
			{
				return exitCommand ?? ( exitCommand = new DelegateCommand(Exit) );
			}
		}

		public MainViewModel()
		{

			InitDataSource();
			InitXamDataGrid();

		}

		private void InitDataSource()
		{

			borderCrossingRepository = new BorderCrossingRepository();

			BorderCrossingCollection = new ObservableCollection<BorderCrossing>(borderCrossingRepository.Get());
			OnPropertyChanged("BorderCrossingCollection");

			BorderCrossingChartingDtoCollection = new ObservableCollection<BorderCrossingChartingDto>();
			OnPropertyChanged("BorderCrossingChartingDtoCollection");

			TestSeriesItemCollection = new ObservableCollection<XamChartSeriesItem>();

		}

		private void InitXamDataGrid()
		{

			InitFieldLayoutSettings();

			InitFieldSettings();

			BuildHeaders();

		}

		private void InitFieldSettings()
		{
			FieldSettings = new FieldSettings
			{
				AllowFixing = AllowFieldFixing.Near,
				AllowResize = true,
				LabelClickAction = LabelClickAction.SelectField,
				CellClickAction = CellClickAction.SelectCell,
			};
		}

		private void InitFieldLayoutSettings()
		{
			FieldLayoutSettings = new FieldLayoutSettings
			{
				AutoGenerateFields = false,
				AutoArrangeCells = AutoArrangeCells.Never,
				FixedFieldUIType = FixedFieldUIType.Splitter,
				AllowFieldMoving = AllowFieldMoving.No,
				HeaderPlacementInGroupBy = HeaderPlacementInGroupBy.OnTopOnly,
				SelectionTypeCell = SelectionType.Extended,
				DataRecordSizingMode = DataRecordSizingMode.SizedToContentAndIndividuallySizable,
				DataErrorDisplayMode = DataErrorDisplayMode.ErrorIcon,
				FilterUIType = FilterUIType.LabelIcons,
				GroupByExpansionIndicatorVisibility = Visibility.Collapsed,
				SummaryDescriptionVisibility = Visibility.Collapsed,
				GroupBySummaryDisplayMode = GroupBySummaryDisplayMode.Default,
				SelectionTypeField = SelectionType.Extended,
				SupportDataErrorInfo = SupportDataErrorInfo.RecordsOnly,
				AllowClipboardOperations = AllowClipboardOperations.All,
				CopyFieldLabelsToClipboard = false,
			};
		}

		private void UpdateBorderCrossingDetails()
		{

			BorderCrossingChartingDto = new BorderCrossingChartingDto(CurrentBorderCrossing);
			OnPropertyChanged("BorderCrossingChartingDto");

			BorderCrossingChartingDtoCollection.Clear();
			TestSeriesItemCollection.Clear();

			var allCrossings = borderCrossingRepository.Get().ToList();

			var crossings =
				allCrossings
					.Where(
						i =>
						i.State == CurrentBorderCrossing.State && i.Port == CurrentBorderCrossing.Port &&
						i.Year == CurrentBorderCrossing.Year
						)
					.OrderBy(i => i.Year)
				;

			foreach ( var dto in crossings.Select(c => new BorderCrossingChartingDto(c)) )
				BorderCrossingChartingDtoCollection.Add(dto);

			var years = allCrossings.Where(
				i =>
				i.State == CurrentBorderCrossing.State &&
				i.Port == CurrentBorderCrossing.Port
				)
				.Select(i => i.Year)
				.Distinct()
				.OrderByDescending(i => i);

			var months = new List<string> { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
			foreach ( var year in years )
			{

				var allCrossingsAtPortByYear = allCrossings
					.Where(i => i.Year == year && i.State == CurrentBorderCrossing.State && i.Port == CurrentBorderCrossing.Port)
					.ToList();

				var tdic = new ObservableCollection<XamChartDataItem>();
				foreach ( var month in months )
				{
					var crossing = allCrossingsAtPortByYear.FirstOrDefault(i => i.Month == month);
					var tdi = new XamChartDataItem(crossing.Month + ", " + year, month, crossing.Trucks);
					tdic.Add(tdi);
				}


				var tsi = new XamChartSeriesItem
				{
					ChartType = ChartType.Line,
					DataMapping = "Label = Label; Value = Value; Tooltip = Tooltip",
					LegendTitle = year.ToString(),
					DataSource = tdic,
			 	};
				TestSeriesItemCollection.Add(tsi);
			}

			OnPropertyChanged("BorderCrossingChartingDtoCollection");
			OnPropertyChanged("ChartDetails");
			OnPropertyChanged("LineSeriesCollection");
			OnPropertyChanged("TestSeriesItemCollection");

		}

		public string VersionMethod()
		{
			var assem = Assembly.GetExecutingAssembly();
			var assemName = assem.GetName();
			var ver = assemName.Version;
			return assemName.Name + ", Version " + ver;
		}

		private void BuildHeaders()
		{

			const int bottom = 2;
			const int middle = 1;
			const int top = 0;

			var hiddenStyle = new Style(typeof(CellValuePresenter));
			hiddenStyle.Setters.Add(new Setter(UIElement.VisibilityProperty, Visibility.Collapsed));

			var fields = new List<Field>();

			// Id, 0
			var field = new Field
			{
				Name = "Id",
				Visibility = Visibility.Collapsed,
				Column = 0,
				Row = bottom,

			};
			fields.Add(field);

			// State 1
			field = new Field
			{
				Name = "State",
				Label = "State",
				Visibility = Visibility.Visible,
				Column = 1,
				Row = bottom,
			};
			fields.Add(field);

			// Port 2
			field = new Field
			{
				Name = "Port",
				Visibility = Visibility.Visible,
				Column = 2,
				Row = bottom,
			};
			fields.Add(field);

			// Year 3
			field = new Field
			{
				Name = "Year",
				Visibility = Visibility.Visible,
				Column = 3,
				Row = bottom,
			};
			fields.Add(field);

			// Month 4
			field = new Field
			{
				Name = "Month",
				Visibility = Visibility.Visible,
				Column = 4,
				Row = bottom,
			};
			fields.Add(field);

			// Vehicles header 5
			var unboundField = new UnboundField
			{
				Row = top,
				Column = 5,
				ColumnSpan = 11,
				Label = "Vehicles",
				Settings =
					{
						CellMaxHeight = 0,
						AllowGroupBy = false,
						CellValuePresenterStyle = hiddenStyle,
					}
			};
			fields.Add(unboundField);

			// Truck header 5
			unboundField = new UnboundField
			{
				Row = middle,
				Column = 5,
				ColumnSpan = 3,
				Label = "Trucks",
				Settings =
				{
					CellMaxHeight = 0,
					AllowGroupBy = false,
					CellValuePresenterStyle = hiddenStyle,
				}

			};
			fields.Add(unboundField);

			//  Trucks 5
			field = new Field
			{
				Name = "Trucks",
				Label = "All Trucks",
				Visibility = Visibility.Visible,
				Column = 5,
				Row = bottom,
			};
			fields.Add(field);

			//  LoadedTrucks 6
			field = new Field
			{
				Name = "LoadedTruckContainers",
				Label = "Loaded",
				Visibility = Visibility.Visible,
				Column = 6,
				Row = bottom,
			};
			fields.Add(field);

			//  EmptyTrucks 7
			field = new Field
			{
				Name = "EmptyTruckContainers",
				Label = "Empty",
				Visibility = Visibility.Visible,
				Column = 7,
				Row = bottom,
			};
			fields.Add(field);

			unboundField = new UnboundField
			{
				Row = middle,
				Column = 8,
				ColumnSpan = 4,
				Label = "Trains",
				Settings =
				{
					CellMaxHeight = 0,
					AllowGroupBy = false,
					CellValuePresenterStyle = hiddenStyle,
				}

			};
			fields.Add(unboundField);

			//  Trains 8
			field = new Field
			{
				Name = "Trains",
				Label = "All Trains",
				Visibility = Visibility.Visible,
				Column = 8,
				Row = bottom,
			};
			fields.Add(field);

			//  LoadedRailContainers 9
			field = new Field
			{
				Name = "LoadedRailContainers",
				Label = "Loaded Containers",
				Visibility = Visibility.Visible,
				Column = 9,
				Row = bottom,
			};
			fields.Add(field);

			//  EmptyRailContainers 10
			field = new Field
			{
				Name = "EmptyRailContainers",
				Label = "Empty Containers",
				Visibility = Visibility.Visible,
				Column = 10,
				Row = bottom,
			};
			fields.Add(field);

			//  TrainPassengers 11
			field = new Field
			{
				Name = "TrainPassengers",
				Label = "Passengers",
				Visibility = Visibility.Visible,
				Column = 11,
				Row = bottom,
			};
			fields.Add(field);

			unboundField = new UnboundField
			{
				Row = middle,
				Column = 12,
				ColumnSpan = 2,
				Label = "Buses",
				Settings =
				{
					CellMaxHeight = 0,
					AllowGroupBy = false,
					CellValuePresenterStyle = hiddenStyle,
				}

			};
			fields.Add(unboundField);

			//  Buses 12
			field = new Field
			{
				Name = "Buses",
				Label = "All",
				Visibility = Visibility.Visible,
				Column = 12,
				Row = bottom,
			};
			fields.Add(field);

			//  BusPassengers 13
			field = new Field
			{
				Name = "BusPassengers",
				Label = "Passengers",
				Visibility = Visibility.Visible,
				Column = 13,
				Row = bottom,
			};
			fields.Add(field);

			unboundField = new UnboundField
			{
				Row = middle,
				Column = 14,
				ColumnSpan = 2,
				Label = "Personal Vehicles",
				Settings =
				{
					CellMaxHeight = 0,
					AllowGroupBy = false,
					CellValuePresenterStyle = hiddenStyle,
				}

			};
			fields.Add(unboundField);

			//  PersonalVehicles 14
			field = new Field
			{
				Name = "PersonalVehicles",
				Label = "All",
				Visibility = Visibility.Visible,
				Column = 14,
				Row = bottom,
			};
			fields.Add(field);

			//  PassengersPersonalVehicles 15
			field = new Field
			{
				Name = "PersonalVehiclePassengers",
				Label = "Passengers",
				Visibility = Visibility.Visible,
				Column = 15,
				Row = bottom,
			};
			fields.Add(field);

			//  Pedestrians 16
			field = new Field
			{
				Name = "Pedestrians",
				Visibility = Visibility.Visible,
				Column = 16,
				Row = bottom,
			};
			fields.Add(field);

			FieldCollection = fields;
		}

		private static void Exit()
		{
			Application.Current.Shutdown();
		}

	}

}
